# -*- coding: utf-8 -*-
# Copyright (c) 2008-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from AccessControl import getSecurityManager

from five import grok
from zope.component import getUtility

from silva.core.interfaces import IVersionedObject
from silva.core.interfaces import IPublicationWorkflow
from silva.core.interfaces import VersioningError
from silva.core.messages.interfaces import IMessageService
from silva.translations import translate as _
from silva.ui.rest import UIREST, SMITransaction
from silva.ui.menu import ActionMenu, MenuItem
from silva.ui.rest.exceptions import RESTRedirectHandler


class PublicationMenuItem(MenuItem):
    """Base menu item for publication actions.
    """
    grok.baseclass()
    grok.adapts(ActionMenu, IVersionedObject)
    grok.require('silva.ChangeSilvaContent')

    def can_approve_content(self):
        sm = getSecurityManager()
        return sm.checkPermission('Approve Silva content', self.content)


class PublicationAction(UIREST):
    grok.baseclass()
    grok.context(IVersionedObject)
    grok.require('silva.ChangeSilvaContent')

    def process(self, workflow):
        raise NotImplementedError

    def send_notification(self, message, type="feedback"):
        service = getUtility(IMessageService)
        service.send(message, self.request, namespace=type)

    def POST(self):
        self.workflow = IPublicationWorkflow(self.context)
        try:
            with SMITransaction(self):
                self.process()
        except VersioningError as error:
            # Notify failure.
            return self.json_response({
                    'content': {},
                    'notifications': [{
                        'message': self.translate(error.reason),
                        'category': 'error',
                        'autoclose': 4000},]})

        # Refresh screen on success
        return RESTRedirectHandler(
            'silva.ui/' + self.request.form.get('screen', 'content'),
            True).publish(self)


class NewVersionMenu(PublicationMenuItem):
    grok.order(100)

    name = _(u'New version')
    description = _(u'Work on a new version while the previous stays online.')
    action = 'newversion'
    accesskey = u'n'
    icon = 'document'

    def available(self):
        return (self.content.get_editable() is None and
                self.content.get_approved_version() is None)


class NewVersionAction(PublicationAction):
    grok.name('silva.ui.actions.newversion')

    def process(self):
        self.workflow.new_version()
        self.send_notification(_(u'New version created.'))


class RequestApprovalMenu(PublicationMenuItem):
    grok.order(10)

    name = _(u'Request approval')
    description = _(u'Request approval for immediate publication.')
    action = 'requestapproval'
    accesskey = u'r'
    icon = 'check'

    def available(self):
        return bool(not self.can_approve_content() and
                    self.content.get_unapproved_version() is not None and
                    not self.content.is_approval_requested())


class RequestApprovalAction(PublicationAction):
    grok.name('silva.ui.actions.requestapproval')

    def process(self):
        self.workflow.request_approval()
        self.send_notification(
            _(u"Approval requested for immediate publication."))


class WithdrawApprovalRequestMenu(PublicationMenuItem):
    grok.order(40)

    name = _(u'Withdraw request')
    description = _(u'Withdraw a request for approval.')
    action = 'withdrawrequest'
    accesskey = u'w'
    icon = 'close'

    def available(self):
        return bool(not self.can_approve_content() and
                    self.content.is_approval_requested())


class WithdrawApprovalRequestAction(PublicationAction):
    grok.name('silva.ui.actions.withdrawrequest')

    def process(self):
        self.workflow.withdraw_request()
        self.send_notification(_(u"Withdrew request for approval."))


class RejectApprovalRequestMenu(PublicationMenuItem):
    grok.order(80)

    name = _(u'Reject request')
    description = _(u'reject a request for approval')
    action = 'rejectrequest'
    accesskey = u'w'
    icon = 'close'

    def available(self):
        return bool(self.can_approve_content() and
                    self.content.is_approval_requested())


class RejectApprovalRequestAction(PublicationAction):
    grok.name('silva.ui.actions.rejectrequest')

    def process(self):
        self.workflow.reject_request()
        self.send_notification(_(u"Rejected request for approval."))


class RevokeApprovalMenu(PublicationMenuItem):
    grok.order(81)

    name = _(u'Revoke approval')
    description = _(u'Un-approve this version in order to edit it.')
    action = 'revokeapproval'
    accesskey = u'r'
    icon = 'cancel'

    def available(self):
        return bool(self.content.get_approved_version())


class RevokeApprovalAction(PublicationAction):
    grok.name('silva.ui.actions.revokeapproval')

    def process(self):
        self.workflow.revoke_approval()
        self.send_notification(_(u'Revoked approval.'))


class PublishMenu(PublicationMenuItem):
    grok.order(10)

    name = _(u'Publish')
    description = _(u'Publish now this item.')
    action = 'publish'
    accesskey = u'p'
    icon = 'check'

    def available(self):
        return bool(self.can_approve_content() and
                    self.content.get_unapproved_version())


class PublishAction(PublicationAction):
    grok.name('silva.ui.actions.publish')

    def process(self):
        self.workflow.publish()
        self.send_notification(_(u"Version published."))





# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest
from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings


class ReaderFolderTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Folder')

    def test_folder_listing(self):
        """A reader cannot create a folder, however he can look into it.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('reader')
        self.assertEqual(browser.inspect.title, u'root')
        self.assertEqual(browser.inspect.tabs, ['Content', 'Preview'])
        self.assertEqual(browser.inspect.activetabs, ['Content'])
        self.assertEqual(browser.inspect.views, ['View...'])
        self.assertEqual(browser.inspect.toolbar, [])

        # We should see our folder.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Folder', 'identifier': 'folder', 'author': 'editor'}])
        # Select the folder.
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(browser.inspect.toolbar, ['Copy'])
        self.assertEqual(browser.inspect.listing[0].goto_dropdown.click(), 200)
        self.assertEqual(browser.inspect.listing[0].goto_actions, ['Preview'])

        # Go inside the folder.
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.title, u'Folder')
        self.assertEqual(browser.inspect.tabs, ['Content', 'Preview'])
        self.assertEqual(browser.inspect.activetabs, ['Content'])
        self.assertEqual(browser.inspect.views, ['View...'])
        self.assertEqual(browser.inspect.listing, [])
        self.assertEqual(browser.inspect.toolbar, [])

        # We go through the tabs.
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Content'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Content'])


class AuthorFolderTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'
    toolbar = []
    access = False

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def test_folder_add_and_listing(self):
        """Create a folder check its tabs and actions and delete it.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(
            browser.inspect.tabs['Add'].name.click(),
            200)
        self.assertIn(
            'Silva Folder',
            browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva Folder'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva Folder"])

        add_form = browser.inspect.form['Add a Silva Folder']
        add_form.fields['id'].value = 'folder'
        add_form.fields['title'].value = 'Test'
        self.assertEqual(add_form.actions, ['Cancel', 'Save'])
        self.assertEqual(add_form.actions['Save'].click(), 200)
        browser.macros.assertFeedback(u"Added Silva Folder.")

        # Inspect new created content.
        self.assertEqual(browser.inspect.title, u"Test")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(
            browser.inspect.views,
            ['View...'])
        self.assertEqual(
            browser.inspect.activetabs,
            ['Content'])
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Test', 'identifier': 'index', 'author': self.user}])

        # We go through the tabs
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Content'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our folder.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Test', 'identifier': 'folder', 'author': self.user}])
        # Select the folder
        self.assertEqual(
            browser.inspect.listing[0].identifier.click(),
            200)
        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename'] + self.toolbar)
        self.assertEqual(
            browser.inspect.listing[0].goto_dropdown.click(),
            200)
        self.assertEqual(
            browser.inspect.listing[0].goto_actions,
            ['Preview', 'Properties'] + (['Access'] if self.access else []))

        # Delete the folder
        self.assertEqual(
            browser.inspect.toolbar['Delete'].click(),
            200)
        # Folder is not deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Test', 'identifier': 'folder', 'author': self.user}])
        self.assertEqual(
            browser.inspect.dialog,
            [{'title': 'Confirm deletion'}])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(
            browser.inspect.listing,
            [])
        browser.macros.assertFeedback(u'Deleted "Test".')

    def test_folder_feeds(self):
        """Test feeds settings. An author doesn't have the right to change them.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Feeds')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Feeds', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)
        self.assertEqual(browser.inspect.title, u'Feeds')
        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertNotIn({'title': u'Atom/rss feeds'}, browser.inspect.form)

        # By default feeds are off
        xml_browser = self.layer.get_browser()
        self.assertEqual(xml_browser.open('/root/folder/atom.xml'), 404)
        self.assertEqual(xml_browser.open('/root/folder/rss.xml'), 404)

    def test_folder_convert(self):
        """Test folder to publication conversion.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)
        self.assertEqual(browser.inspect.title, u'Data')
        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertNotIn({'title': u'Container type'}, browser.inspect.form)


class EditorFolderTestCase(AuthorFolderTestCase):
    user = 'editor'
    toolbar = ['Publish']

    def test_folder_feeds(self):
        """Test feeds settings. An editor and above can change them
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Feeds')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Feeds', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)
        self.assertEqual(browser.inspect.title, u'Feeds')
        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)

        self.assertIn({'title': u'Atom/rss feeds'}, browser.inspect.form)
        settings = browser.inspect.form['feeds']
        self.assertEqual(settings.title, 'Atom/rss feeds')
        self.assertEqual(settings.form.inspect.fields, [u'Allow feeds'])
        self.assertEqual(settings.actions, [u'Change feed settings'])
        self.assertEqual(
            settings.form.inspect.fields['Allow feeds'].checked,
            False)

        settings.form.inspect.fields['Allow feeds'].checked = True
        self.assertEqual(settings.actions['Change feed settings'].click(), 200)
        browser.macros.assertFeedback('Feed settings saved.')

        self.assertIn({'title': u'Atom/rss feeds'}, browser.inspect.form)
        settings = browser.inspect.form['feeds']
        self.assertEqual(settings.form.inspect.fields, [u'Allow feeds'])
        self.assertEqual(settings.actions, [u'Change feed settings'])
        self.assertEqual(
            settings.form.inspect.fields['Allow feeds'].checked,
            True)

        # Feeds are now available.
        xml_browser = self.layer.get_browser()
        self.assertEqual(xml_browser.open('/root/folder/atom.xml'), 200)
        self.assertEqual(xml_browser.content_type, 'text/xml;charset=UTF-8')
        self.assertEqual(xml_browser.open('/root/folder/rss.xml'), 200)
        self.assertEqual(xml_browser.content_type, 'text/xml;charset=UTF-8')

    def test_folder_convert(self):
        """Test folder to publication conversion.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)
        self.assertEqual(browser.inspect.title, u'Data')
        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertIn({'title': u'Container type'}, browser.inspect.form)

        convert = browser.inspect.form['Container type']
        self.assertEqual(convert.title, 'Container type')
        self.assertEqual(convert.form.inspect.fields, [])
        self.assertEqual(convert.actions, ['Convert to publication'])
        self.assertEqual(convert.actions['Convert to publication'].click(), 200)
        browser.macros.assertFeedback("Changed into publication.")

        self.assertEqual(browser.inspect.title, u'Data')
        convert = browser.inspect.form['Container type']
        self.assertEqual(convert.title, 'Container type')
        self.assertEqual(convert.form.inspect.fields, [])
        self.assertNotIn('Convert to publication', convert.actions)

    def test_folder_access(self):
        """Test folder access tab.

        Editors don't have access to the access tab.
        """
        self.assertEqual(self.access, False)
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)

        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].open.click(), 200)
        self.assertNotIn('Access', browser.inspect.tabs['Settings'].entries)

    def test_folder_addables(self):
        """Test folder addable tab.

        Editors don't have access to the addable tab.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)

        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].open.click(), 200)
        self.assertNotIn('Addables', browser.inspect.tabs['Settings'].entries)

    def test_folder_customization(self):
        """Test folder customization.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].open.click(), 200)
        self.assertIn('Customization', browser.inspect.tabs['Settings'].entries)
        self.assertEqual(
            browser.inspect.tabs['Settings'].entries['Customization'].click(),
            200)

        self.assertEqual(
            browser.inspect.form,
            [u'Interfaces in use which affect the behavior of the item',
             u'Add a marker to alter the behavior'])

        form = browser.inspect.form['Add a marker to alter the behavior']
        self.assertEqual(form.actions, ['Add marker'])


class ChiefEditorFolderTestCase(EditorFolderTestCase):
    user = 'chiefeditor'
    access = True

    def test_folder_addables(self):
        """Test folder addable tab.

        Editors don't have access to the addable tab.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)

        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].open.click(), 200)
        self.assertIn('Addables', browser.inspect.tabs['Settings'].entries)
        self.assertEqual(
            browser.inspect.tabs['Settings'].entries['Addables'].click(),
            200)

        self.assertEqual(browser.inspect.form, ['Addable settings'])
        form = browser.inspect.form['Addable settings']
        self.assertIn('Acquire', form.fields)
        self.assertEqual(form.fields['Acquire'].checked, True)
        self.assertEqual(form.actions, ['Cancel', 'Save addables settings'])

        # Restrict the settings
        form.fields['Acquire'].checked = False
        form.fields['form.field.addables'].value = ['Silva Link']
        self.assertEqual(form.actions['Save addables settings'].click(), 200)
        browser.macros.assertFeedback(
            u'Changes to addables content types saved.')

        self.assertIn('Add', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertEqual(browser.inspect.tabs['Add'].entries, ['Silva Link'])
        self.assertNotIn('Silva Folder', browser.inspect.tabs['Add'].entries)

        # Now acquire settings
        self.assertEqual(browser.inspect.form, ['Addable settings'])
        form = browser.inspect.form['Addable settings']
        form.fields['Acquire'].checked = True
        self.assertEqual(form.actions['Save addables settings'].click(), 200)
        browser.macros.assertFeedback(
            u'Addable settings are now aquired.')
        self.assertIn('Add', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn('Silva Folder', browser.inspect.tabs['Add'].entries)

    def test_folder_access(self):
        """Test folder access tab.

        Editors don't have access to the access tab.
        """
        self.assertEqual(self.access, True)
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'folder', 'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].open.click(), 200)
        self.assertIn('Access', browser.inspect.tabs['Settings'].entries)
        self.assertEqual(
            browser.inspect.tabs['Settings'].entries['Access'].click(),
            200)

        # Test restriction form
        self.assertEqual(
            browser.inspect.form,
            ['User clipboard', 'User roles', 'Public view access restriction'])
        form = browser.inspect.form['Public view access restriction']
        self.assertIn('Minimum role', form.fields)
        self.assertEqual(form.fields['Minimum role'].value, u'None')
        self.assertEqual(
            form.fields['Minimum role'].options,
            [u'None', u'Authenticated', u'Viewer', u'Viewer +', u'Viewer ++',
             u'Reader', u'Author', u'Editor', u'ChiefEditor', u'Manager'])
        form.fields['Minimum role'].value = 'Editor'
        self.assertEqual(form.actions, ['Cancel', 'Set restriction'])
        self.assertEqual(form.actions['Set restriction'].click(), 200)
        browser.macros.assertFeedback(
            u'The minimum required role to access this item has been set to "Editor".')

        # Now the restriction is set, and a new action is available
        self.assertEqual(
            browser.inspect.form,
            ['User clipboard', 'User roles', 'Public view access restriction'])
        form = browser.inspect.form['Public view access restriction']
        self.assertIn('Minimum role', form.fields)
        self.assertEqual(form.fields['Minimum role'].value, u'Editor')
        self.assertEqual(
            form.actions,
            ['Cancel', 'Acquire restriction', 'Set restriction'])
        self.assertEqual(form.actions['Acquire restriction'].click(), 200)
        browser.macros.assertFeedback(
            u'Now acquiring the minimum role setting.')


class ManagerFolderTestCase(ChiefEditorFolderTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReaderFolderTestCase))
    suite.addTest(unittest.makeSuite(AuthorFolderTestCase))
    suite.addTest(unittest.makeSuite(EditorFolderTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorFolderTestCase))
    suite.addTest(unittest.makeSuite(ManagerFolderTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from setuptools import setup, find_packages
import os

version = '3.0.3'

tests_require = [
    'Products.Silva [test]',
    ]

setup(name='silva.core.smi',
      version=version,
      description="Silva Management Interface",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Zope2",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='silva zope2 smi',
      author='Infrae',
      author_email='info@infrae.com',
      url='https://github.com/silvacms/silva.core.smi',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['silva', 'silva.core'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'AccessControl',
        'five.grok',
        'infrae.comethods',
        'lxml',
        'grokcore.chameleon',
        'setuptools',
        'silva.core.cache',
        'silva.core.conf',
        'silva.core.interfaces',
        'silva.core.layout',
        'silva.core.messages',
        'silva.core.references',
        'silva.core.services',
        'silva.core.views',
        'silva.fanstatic',
        'silva.translations',
        'silva.ui',
        'z3c.schema',
        'zeam.form.autofields',
        'zeam.form.silva',
        'zeam.form.ztk',
        'zope.cachedescriptors',
        'zope.component',
        'zope.interface',
        'zope.intid',
        'zope.schema',
        'zope.traversing',
        'zope.publisher',
        ],
      tests_require=tests_require,
      extras_require={'test': tests_require},
      )



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt
import unittest
import zipfile, os
from datetime import datetime
from lxml import etree
from StringIO import StringIO

from Products.Silva.testing import FunctionalLayer

NS = {
    'silva' : 'http://infrae.com/namespace/silva'
}


class TestExport(unittest.TestCase):

    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.browser = self.layer.get_browser()
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addPublication('publication', 'Publication')

    def test_export_form(self):
        self.browser.login('editor')
        status = self.browser.open(
            'http://localhost/root/edit/tab_status_export')
        self.assertEquals(200, status)

    def test_simple_export(self):
        self.browser.login('editor')
        status = self.browser.open(
            'http://localhost/root/edit/tab_status_export')
        self.assertEquals(200, status)
        form = self.browser.get_form('form')
        status = form.get_control('form.action.export').click()
        self.assertEquals(200, status)
        self.assertEquals(
            'application/octet-stream', self.browser.content_type)
        filename = 'root_export_%s.zip' % datetime.now().strftime('%Y-%m-%d')
        self.assertEquals('attachment;filename=%s' % filename,
                          self.browser.headers['content-disposition'])
        fd = StringIO(self.browser.contents)
        zdata = zipfile.ZipFile(fd, 'r')
        self.assertEquals(['silva.xml'], zdata.namelist())
        zdata.testzip()
        silvaxml = zdata.extract('silva.xml')
        try:
            tree = etree.parse(silvaxml)
            self.assertTrue(tree.xpath(
                '//silva:publication[@id="root"]', namespaces=NS))
            self.assertFalse(tree.xpath(
                '//silva:publication[@id="publication"]', namespaces=NS))
        finally:
            os.unlink(silvaxml)

    def test_export_form_with_sub_publication(self):
        self.browser.login('editor')
        status = self.browser.open(
            'http://localhost/root/edit/tab_status_export')
        self.assertEquals(200, status)
        form = self.browser.get_form('form')
        include_cbx = form.get_control('form.field.include_sub_publications')
        include_cbx.checked = True
        status = form.get_control('form.action.export').click()
        self.assertEquals(200, status)
        self.assertEquals(
            'application/octet-stream', self.browser.content_type)
        filename = 'root_export_%s.zip' % datetime.now().strftime('%Y-%m-%d')
        self.assertEquals('attachment;filename=%s' % filename,
                          self.browser.headers['content-disposition'])
        fd = StringIO(self.browser.contents)
        zdata = zipfile.ZipFile(fd, 'r')
        self.assertEquals(['silva.xml'], zdata.namelist())
        zdata.testzip()
        silvaxml = zdata.extract('silva.xml')
        try:
            tree = etree.parse(silvaxml)
            self.assertTrue(
                tree.xpath('//silva:publication[@id="root"]',
                           namespaces=NS))
            self.assertTrue(
                tree.xpath('//silva:publication[@id="publication"]',
                           namespaces=NS))
        finally:
            os.unlink(silvaxml)


def test_suite():
    suite = unittest.TestSuite()
    return suite
    suite.addTest(unittest.makeSuite(TestExport))



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok

from silva.core.interfaces.errors import ExternalReferenceError
from silva.translations import translate as _
from silva.core.views.views import Page


class ExternalReferenceErrorPage(Page):
    """ Page to render broken references errors.

    It redirects to break references form if the user as the necessary rights to
    break references.
    """
    grok.context(ExternalReferenceError)
    grok.name('error.html')

    title = _('External reference exported')

    def update(self):
        self.origin = self.context.error.content.get_silva_object()
        self.target = self.context.error.target.get_silva_object()
        self.exported = self.context.error.exported.get_silva_object()



# -*- coding: utf-8 -*-
# Copyright (c) 2013  Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.ftesting import smi_settings
from Products.Silva.testing import FunctionalLayer, Transaction


class AuthorFolderTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'
    toolbar = ['Request approval']

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def test_content_cut(self):
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addFolder('folder', 'Folder')
            factory.manage_addMockupVersionedContent('document', 'Document')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(browser.inspect.title, 'root')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Folder',
              'identifier': 'folder',
              'author': u'editor'},
             {'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.toolbar, [])
        self.assertEqual(browser.inspect.listing[1].identifier.click(), 200)

        self.assertEqual(
            browser.inspect.toolbar,
            [u'Cut', u'Copy', u'Delete', u'Rename'] + self.toolbar)
        self.assertEqual(browser.inspect.toolbar['Cut'].click(), 200)
        browser.macros.assertFeedback('Cut 1 item(s) in the clipboard.')

        # Go in the folder and past the document.
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.title, 'Folder')

        self.assertEqual(browser.inspect.toolbar, ['Paste'])
        self.assertEqual(browser.inspect.toolbar['Paste'].click(), 200)
        browser.macros.assertFeedback('Moved "Document".')

        # The document is now in the folder.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])

        # The document is gone from the root
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, 'root')
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Folder',
              'identifier': 'folder',
              'author': self.user}])

    def test_content_copy(self):
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(browser.inspect.title, 'root')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.toolbar, [])
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)

        self.assertEqual(
            browser.inspect.toolbar,
            [u'Cut', u'Copy', u'Delete', u'Rename'] + self.toolbar)
        self.assertEqual(browser.inspect.toolbar['Copy'].click(), 200)
        browser.macros.assertFeedback('Copied 1 item(s) in the clipboard.')

        self.assertEqual(
            browser.inspect.toolbar,
            [u'Cut', u'Copy', u'Paste', u'Delete', u'Rename'] + self.toolbar)
        self.assertEqual(browser.inspect.toolbar['Paste'].click(), 200)
        browser.macros.assertFeedback('Pasted as a copy "Document".')

        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'},
              {'title': u'Document',
               'identifier': 'copy_of_document',
               'author': self.user}])


class EditorFolderTestCase(AuthorFolderTestCase):
    user = 'editor'
    toolbar = ['Publish']


class ChiefEditorFolderTestCase(EditorFolderTestCase):
    user = 'chiefeditor'


class ManagerFolderTestCase(ChiefEditorFolderTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AuthorFolderTestCase))
    suite.addTest(unittest.makeSuite(EditorFolderTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorFolderTestCase))
    suite.addTest(unittest.makeSuite(ManagerFolderTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import bisect

from five import grok
from zope.component import getUtility, getMultiAdapter
from zope.cachedescriptors.property import Lazy

from silva.core.interfaces import IIconResolver, IAuthorizationManager
from silva.core.interfaces import ISilvaObject, IVersion, IVersionedObject
from silva.core.references.interfaces import IReferenceService
from silva.core.services.interfaces import IMetadataService
from silva.core.views import views as silvaviews
from silva.core.views.interfaces import IContentURL
from silva.translations import translate as _
from silva.ui.menu import ContentMenu, MenuItem
from silva.ui.rest import Screen
from zeam.form import silva as silvaforms

from Products.Silva.Security.management import is_role_greater_or_equal


class Properties(silvaforms.SMIComposedForm):
    """ Properties tab allows metadata editing.
    """
    grok.adapts(Screen, ISilvaObject)
    grok.name('properties')
    grok.require('silva.ChangeSilvaContent')
    label = _('Properties')


class PropertiesMenu(MenuItem):
    grok.adapts(ContentMenu, ISilvaObject)
    grok.order(20)
    grok.require('silva.ChangeSilvaContent')
    name = _('Properties')
    screen = Properties


class MetadataFormGroup(silvaforms.SMISubFormGroup):
    """Form group that holds metadata forms.
    """
    grok.context(ISilvaObject)
    grok.view(Properties)
    grok.order(10)
    # metadata category filter
    category = ''


class BindingCache(object):

    def __init__(self, binding):
        self.binding = binding
        self.clear()

    def clear(self):
        self._cache = {}

    def get_element(self, set_name, element_name):
        id = (set_name, element_name)
        element = self._cache.get(id)
        if element is not None:
            return element
        self._cache[id]= element = \
            self.binding.getElement(set_name, element_name)
        return element


class MetadataForm(silvaforms.SMISubForm):
    grok.baseclass()
    grok.context(ISilvaObject)
    grok.view(MetadataFormGroup)
    grok.template('metadataform')
    grok.require('zope2.ManageProperties')

    label = _('Editable item properties')

    @Lazy
    def binding(self):
        service = getUtility(IMetadataService)
        return service.getMetadata(self.getContent())

    def available(self):
        if self.binding is None:
            return False
        return len(self.binding.getSetNames(category=self.parent.category)) != 0

    def update(self):
        if not self.available():
            return
        self.category = self.parent.category
        self.binding_cache = BindingCache(self.binding)
        self.aquired_items = self.binding.listAcquired()
        self.user_role = IAuthorizationManager(self.context).get_user_role()
        self.errors = False
        self.set_names = self.binding.getSetNames(category=self.category)

    def should_display_label(self):
        # Display the title only if there is more than one subform.
        return len(self.parent.subforms) > 1

    def get_set_title(self, set_name):
        return self.binding.getSet(set_name).getTitle() or set_name

    def get_set_description(self, set_name):
        return self.binding.getSet(set_name).getDescription()

    def get_element_names(self, set_name):
        return self.binding.getElementNames(set_name, mode='view')

    def get_element_title(self, set_name, element_name):
        element = self.binding_cache.get_element(set_name, element_name)
        return element.Title()

    def get_element_description(self, set_name, element_name):
        element = self.binding_cache.get_element(set_name, element_name)
        return element.Description()

    def is_element_required(self, set_name, element_name):
        element = self.binding_cache.get_element(set_name, element_name)
        return element.isRequired()

    def is_element_hidden(self, set_name, element_name):
        element = self.binding_cache.get_element(set_name, element_name)
        return bool(element.field.get_value('hidden'))

    def is_element_acquireable(self, set_name, element_name):
        element = self.binding_cache.get_element(set_name, element_name)
        return element.isAcquireable()

    def is_element_editable(self, set_name, element_name):
        if not is_role_greater_or_equal(self.user_role, 'Author'):
            return False
        return self.binding.isEditable(set_name, element_name)

    def is_element_acquired(self, set_name, element_name):
        if (set_name, element_name) in self.aquired_items:
            return True
        return False

    def render_element(self, set_name, element_name):
        if self.is_element_editable(set_name, element_name):
            if self.errors:
                return self.binding.renderElementEdit(
                    set_name, element_name, self.request)
            else:
                return self.binding.renderElementEdit(set_name, element_name)
        return self.binding.renderElementView(set_name, element_name)

    def render_element_view(self, set_name, element_name):
        return self.binding.renderElementView(set_name, element_name)

    def html_id(self, set_name, element_name):
        element = self.binding_cache.get_element(set_name, element_name)
        return getattr(element.field, 'html_id', None)

    def is_allowed(self, set_name):
        minimal_role = self.binding.getSet(set_name).getMinimalRole()
        if not minimal_role:
            return True
        if is_role_greater_or_equal(self.user_role, minimal_role):
            return True
        return False


class MetadataEditForm(MetadataForm):
    grok.baseclass()
    grok.view(MetadataFormGroup)
    actions = silvaforms.Actions(silvaforms.CancelAction())

    def __init__(self, context, parent, request):
        super(MetadataEditForm, self).__init__(context, parent, request)
        # Prefix is set by grokker and we need to extend it because sometimes
        # when need to use several instances of the same form.
        if self.parent.category:
            self.prefix += '-' + self.parent.category

    @silvaforms.action(
        _('Save changes'),
        implements=silvaforms.IDefaultAction,
        available=lambda form: not form.binding.read_only,
        accesskey='ctrl+s')
    def save(self):
        self.errors = self.binding.setValuesFromRequest(
            self.request, reindex=1)
        if self.errors:
            self.send_message(_(
            'The data that was submitted did not '
            'validate properly.  Please adjust '
            'the form values and submit again.'), type='error')
            return silvaforms.FAILURE
        self.send_message(_('Metadata saved.'), type='feedback')
        return silvaforms.SUCCESS


class MetadataReadOnlyForm(MetadataForm):
    grok.baseclass()
    grok.view(MetadataFormGroup)
    mode = silvaforms.DISPLAY
    actions = silvaforms.Actions(silvaforms.CancelAction())

    def is_element_editable(self, set_name, element_name):
        return False


class EditableMetadataForm(MetadataEditForm):
    """Metadata of the editable version.
    """
    grok.context(IVersionedObject)
    grok.view(MetadataFormGroup)
    grok.order(10)

    def getContent(self):
        return self.context.get_editable()

    def available(self):
        return (bool(self.context.get_unapproved_version()) and
                super(EditableMetadataForm, self).available())


class PreviewableMetadataForm(MetadataEditForm):
    """metadata of previewable version.
    """
    grok.context(IVersionedObject)
    grok.view(MetadataFormGroup)
    grok.order(20)
    label = _('Approved version item properties')

    def getContent(self):
        return self.context.get_previewable()

    def available(self):
        return (bool(self.context.get_approved_version()) and
                super(PreviewableMetadataForm, self).available())


class ViewableMetadataForm(MetadataReadOnlyForm):
    """metadata of the viewable version
    """
    grok.context(IVersionedObject)
    grok.order(30)
    label = _('Public version item properties')

    def getContent(self):
        return self.context.get_viewable()

    def available(self):
        return (bool(self.context.get_public_version()) and
                super(ViewableMetadataForm, self).available())


class LastClosedMetadataForm(MetadataReadOnlyForm):
    """Metadata of the last closed version
    """
    grok.context(IVersionedObject)
    grok.order(30)
    label = _('Last closed version item properties')

    def getContent(self):
        version_id = self.context.get_last_closed_version()
        if version_id is None:
            return None
        return self.context._getOb(version_id)

    def available(self):
        return (not bool(self.context.get_next_version()) and
                not bool(self.context.get_public_version()) and
                super(LastClosedMetadataForm, self).available())


class NonVersionedContentMetadataForm(MetadataEditForm):
    """Metadata for non versioned content
    """
    grok.context(ISilvaObject)
    grok.order(40)

    def getContent(self):
        return self.context

    def available(self):
        return (not IVersionedObject.providedBy(self.context) and
                super(NonVersionedContentMetadataForm, self).available())


class ContentReferencedBy(silvaviews.Viewlet):
    """Report reference usage of this publishable
    """
    grok.template('contentreferencedby')
    grok.context(ISilvaObject)
    grok.view(Properties)
    grok.viewletmanager(silvaforms.SMIFormPortlets)

    def update(self):
        references = {}
        service = getUtility(IReferenceService)
        get_icon = IIconResolver(self.request).get_tag
        for reference in service.get_references_to(self.context):
            source = reference.source
            source_versions = []
            if IVersion.providedBy(source):
                source_versions.append(source.id)
                source = source.get_silva_object()

            url = getMultiAdapter((source, self.request), IContentURL)
            edit_url = str(url) + '/edit'
            if edit_url in references and source_versions:
                previous_versions = references[edit_url]['versions']
                if previous_versions[-1] > source_versions[0]:
                    bisect.insort_right(
                        previous_versions, source_versions[0])
                    continue
                else:
                    source_versions = previous_versions + source_versions

            source_title = source.get_title_or_id()
            references[edit_url] = {
                'title': source_title,
                'path': self.view.get_content_path(source),
                'icon': get_icon(source),
                'versions': source_versions}

        self.references = references.values()
        self.references.sort(key=lambda info: info['title'].lower())

        for info in self.references:
            if info['versions']:
                info['title'] += ' (' + ', '.join(info['versions']) + ')'





# -*- coding: utf-8 -*-
# Copyright (c) 2013  Infrae. All rights reserved.
# See also LICENSE.txt

import unittest
import json
from Products.Silva.testing import FunctionalLayer


class TestRESTIDValidation(unittest.TestCase):

    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()

        factory = self.root.manage_addProduct['Silva']
        factory.manage_addFolder('folder', 'Folder')
        self.folder = self.root._getOb('folder')
        self.assertIsNot(None, self.folder)

        factory = self.folder.manage_addProduct['Silva']
        factory.manage_addMockupVersionedContent('existing', 'Existing')
        self.content = self.folder._getOb('existing')
        self.assertIsNot(None, self.content)

    def test_id_valid(self):
        with self.layer.get_browser() as browser:
            browser.options.handle_errors = False
            browser.login('author')
            status = browser.open(
                'http://localhost/root/folder/++rest++silva.ui/adding/'
                'Silva Publication/++rest++zeam.form.silva.validate',
                form={'addform.field.id': 'notexisting',
                      'prefix.field': 'addform.field.id',
                      'prefix.form': 'addform'},
                method='POST')
            self.assertEqual(200, status)
            self.assertEqual('application/json', browser.content_type)
            data = json.loads(browser.contents)
            self.assertEqual(data['success'], True)

    def test_id_exists(self):
        with self.layer.get_browser() as browser:
            browser.options.handle_errors = False
            browser.login('author')
            status = browser.open(
                'http://localhost/root/folder/++rest++silva.ui/adding/'
                'Silva Publication/++rest++zeam.form.silva.validate',
                form={'addform.field.id': 'existing',
                      'prefix.field': 'addform.field.id',
                      'prefix.form': 'addform'},
                method='POST')
            self.assertEqual(200, status)
            self.assertEqual('application/json', browser.content_type)
            data = json.loads(browser.contents)
            self.assertEqual(data['success'], False)
            self.assertEqual(
                'There is already an object with the'
                ' id existing in this container.',
                data['errors']['addform.field.id'])

    def test_id_invalid(self):
        with self.layer.get_browser() as browser:
            browser.options.handle_errors = False
            browser.login('author')
            status = browser.open(
                'http://localhost/root/folder/++rest++silva.ui/adding/'
                'Silva Publication/++rest++zeam.form.silva.validate',
                form={'addform.field.id': 'in  v alid',
                      'prefix.field': 'addform.field.id',
                      'prefix.form': 'addform'},
                method='POST')
            self.assertEqual(200, status)
            self.assertEqual('application/json', browser.content_type)
            data = json.loads(browser.contents)
            self.assertEqual(data['success'], False)
            self.assertTrue(data['errors']['addform.field.id'].startswith(
                'The id contains strange characters.'))


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestRESTIDValidation))
    return suite




# -*- coding: utf-8 -*-
# Copyright (c) 2008-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.ftesting import smi_settings
from Products.Silva.testing import FunctionalLayer, Transaction
from silva.core.interfaces import IPublicationWorkflow


class PublicationTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')

    def test_author_request_approval_and_approve(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # An author should not see Publish new version form, but he
        # should see Request approval new version.
        self.assertEqual(
            browser.inspect.form,
            [u'Request approval for new version',
             u'Manage versions'])

        form = browser.inspect.form[u'Request approval for new version']
        self.assertEqual(form.actions, ['Request approval'])
        self.assertEqual(form.actions['Request approval'].click(), 200)
        browser.macros.assertFeedback(u"Approval requested.")

        browser.logout()
        browser.login('editor')
        # Select document (author change to author)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'author'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # An editor should see Publish new version, Reject approval
        # request, and Manage versions
        self.assertEqual(
            browser.inspect.form,
            [u'Publish new version',
             u'Reject approval request',
             u'Manage versions'])
        form = browser.inspect.form[u'Publish new version']
        self.assertEqual(form.actions, ['Approve for future', 'Publish now'])
        self.assertEqual(form.actions['Publish now'].click(), 200)
        browser.macros.assertFeedback(u"Version published.")

        # Now only those forms are available
        self.assertEqual(
            browser.inspect.form,
            [u'Manual close',
             u'Manage versions'])

        browser.logout()
        browser.login('author')
        # Select document (author change to author)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # Author should only see those forms
        self.assertEqual(
            browser.inspect.form,
            [u'Manage versions'])

    def test_author_request_approval_and_reject(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # An author should not see Publish new version form, but he
        # should see Request approval new version.
        self.assertEqual(
            browser.inspect.form,
            [u'Request approval for new version', u'Manage versions'])

        form = browser.inspect.form[u'Request approval for new version']
        self.assertEqual(form.actions, ['Request approval'])
        self.assertEqual(form.actions['Request approval'].click(), 200)
        browser.macros.assertFeedback(u"Approval requested.")

        browser.logout()
        browser.login('editor')
        # Select document (author change to author)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'author'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # An editor should see Publish new version, Reject approval
        # request, and Manage versions
        self.assertEqual(
            browser.inspect.form,
            [u'Publish new version',
             u'Reject approval request',
             u'Manage versions'])
        form = browser.inspect.form[u'Reject approval request']
        self.assertEqual(form.actions, ['Reject approval request'])
        self.assertEqual(form.actions['Reject approval request'].click(), 200)
        browser.macros.assertFeedback(u"Rejected request for approval.")

        # Now reject version form is no longer available
        self.assertEqual(
            browser.inspect.form,
            [u'Publish new version',
             u'Manage versions'])

    def test_author_request_approval_and_withdraw(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # An author should not see Publish new version form, but he
        # should see Request approval new version.
        self.assertEqual(
            browser.inspect.form,
            [u'Request approval for new version',
             u'Manage versions'])

        form = browser.inspect.form[u'Request approval for new version']
        self.assertEqual(form.actions, ['Request approval'])
        self.assertEqual(form.actions['Request approval'].click(), 200)
        browser.macros.assertFeedback(u"Approval requested.")

        # Request approval form should no longer be here.
        self.assertEqual(
            browser.inspect.form,
            [u'Withdraw approval request',
             u'Manage versions'])
        form = browser.inspect.form[u'Withdraw approval request']
        self.assertEqual(
            form.fields,
            ['Add message on withdrawal'])
        self.assertEqual(form.actions, ['Withdraw approval request'])
        self.assertEqual(form.actions['Withdraw approval request'].click(), 200)
        browser.macros.assertFeedback(u"Withdrew request for approval.")

    def test_editor_publish_and_manual_close(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('editor')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # You can publish or manage versions
        self.assertEqual(
            browser.inspect.form,
            [u'Publish new version',
             u'Manage versions'])
        form = browser.inspect.form[u'Publish new version']
        self.assertEqual(form.actions, ['Approve for future', 'Publish now'])
        self.assertEqual(form.actions['Publish now'].click(), 200)
        browser.macros.assertFeedback(u"Version published.")

        # You can now manually close the version
        self.assertEqual(
            browser.inspect.form,
            [u'Manual close',
             u'Manage versions'])
        form = browser.inspect.form[u'Manual close']
        self.assertEqual(form.actions, ['Close published version'])
        self.assertEqual(form.actions['Close published version'].click(), 200)
        browser.macros.assertFeedback(u"Version closed.")

        # You can now republish the closed version, if you want.
        self.assertEqual(
            browser.inspect.form,
            [u'Publish closed version',
             u'Manage versions'])
        form = browser.inspect.form[u'Publish closed version']
        self.assertEqual(form.actions, ['Publish now'])
        self.assertEqual(form.actions['Publish now'].click(), 200)
        browser.macros.assertFeedback(u"Version published.")

        # And you end up in the same state than before
        self.assertEqual(
            browser.inspect.form,
            [u'Manual close',
             u'Manage versions'])


class ManageVersionTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')

            # Create a couple of test versions
            IPublicationWorkflow(self.root.document).publish()
            IPublicationWorkflow(self.root.document).new_version()
            IPublicationWorkflow(self.root.document).publish()
            IPublicationWorkflow(self.root.document).new_version()
            IPublicationWorkflow(self.root.document).publish()

    def test_editor_copy_for_editing(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('editor')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # You can publish or manage versions
        self.assertEqual(
            browser.inspect.form,
            [u'Manual close',
             u'Manage versions'])

        form = browser.inspect.form[u'Manage versions']
        self.assertEqual(
            form.actions,
            [u'Delete', u'Copy for editing', u'View', u'Compare'])
        self.assertEqual(len(form.fields), 4)
        self.assertEqual(form.actions['Copy for editing'].click(), 200)
        browser.macros.assertError(u"Please select one version to copy.")

        form = browser.inspect.form[u'Manage versions']
        form.fields['1'].checked = True
        self.assertEqual(form.actions['Copy for editing'].click(), 200)
        browser.macros.assertFeedback(u"Reverted to previous version.")

    def test_editor_view_previous_version(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('editor')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # You can publish or manage versions
        self.assertEqual(
            browser.inspect.form,
            [u'Manual close',
             u'Manage versions'])

        form = browser.inspect.form[u'Manage versions']
        self.assertEqual(
            form.actions,
            [u'Delete', u'Copy for editing', u'View', u'Compare'])
        self.assertEqual(len(form.fields), 4)
        self.assertEqual(form.actions['View'].click(), 200)
        browser.macros.assertError(u"Please select one version to view it.")

        form = browser.inspect.form[u'Manage versions']
        form.fields['1'].checked = True
        self.assertEqual(form.actions['View'].click(), 200)

    def test_editor_delete_version(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('editor')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # You can publish or manage versions
        self.assertEqual(
            browser.inspect.form,
            [u'Manual close',
             u'Manage versions'])

        form = browser.inspect.form[u'Manage versions']
        self.assertEqual(
            form.actions,
            [u'Delete', u'Copy for editing', u'View', u'Compare'])
        self.assertEqual(len(form.fields), 4)
        self.assertEqual(form.actions['Delete'].click(), 200)
        self.assertEqual(browser.inspect.dialog, ['Delete a version'])

        dialog = browser.inspect.dialog['Delete a version']
        self.assertEqual(dialog.buttons, ['Cancel', 'Continue'])
        self.assertEqual(dialog.buttons['Continue'].click(), 200)

        browser.macros.assertError(u"No version selected, nothing deleted.")

        form = browser.inspect.form[u'Manage versions']
        form.fields['1'].checked = True
        self.assertEqual(form.actions['Delete'].click(), 200)
        self.assertEqual(browser.inspect.dialog, ['Delete a version'])

        dialog = browser.inspect.dialog['Delete a version']
        self.assertEqual(dialog.buttons, ['Cancel', 'Continue'])
        self.assertEqual(dialog.buttons['Continue'].click(), 200)

        browser.macros.assertFeedback(u"Version(s) deleted.")

        form = browser.inspect.form[u'Manage versions']
        self.assertEqual(
            form.actions,
            [u'Delete', u'Copy for editing', u'View', u'Compare'])
        self.assertEqual(len(form.fields), 3)
        self.assertNotIn('1', form.fields)
        form.fields['0'].checked = True
        form.fields['2'].checked = True
        self.assertEqual(form.actions['Delete'].click(), 200)
        self.assertEqual(browser.inspect.dialog, ['Delete a version'])

        dialog = browser.inspect.dialog['Delete a version']
        self.assertEqual(dialog.buttons, ['Cancel', 'Continue'])
        self.assertEqual(dialog.buttons['Continue'].click(), 200)

        browser.macros.assertError(u"Cannot delete all versions.")

    def test_author_do_not_delete_version(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        # Select document
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Publish' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Publish'].name.click(), 200)

        # You can publish or manage versions
        self.assertEqual(
            browser.inspect.form,
            [u'Manage versions'])

        form = browser.inspect.form[u'Manage versions']
        self.assertEqual(
            form.actions,
            [u'Copy for editing', u'View', u'Compare'])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PublicationTestCase))
    suite.addTest(unittest.makeSuite(ManageVersionTestCase))
    return suite





# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from datetime import datetime

from five import grok
from zope import schema, interface
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from DateTime import DateTime
from AccessControl.security import checkPermission

from silva.core.interfaces import IContainer
from silva.core.interfaces import IVersioning, IVersionedObject
from silva.core.interfaces import IPublicationWorkflow
from silva.core.interfaces import IRequestForApprovalStatus
from silva.core.interfaces.errors import VersioningError
from silva.core.smi.content.publish import Publish, IPublicationFields
from silva.translations import translate as _

from zeam.form import autofields
from zeam.form import silva as silvaforms
from zeam.form.silva.interfaces import IRemoverAction
from zeam.form.ztk.actions import EditAction


class IPublicationInformation(interface.Interface):
    publication_datetime = schema.Datetime(
        title=_('Publication time'),
        required=True)
    expiration_datetime = schema.Datetime(
        title=_('Expiration time'),
        description=_('Empty indicates no expiration.'),
        required=False)

    @interface.invariant
    def check_publication_expiration(data):
        if data.expiration_datetime:
            if data.expiration_datetime < data.publication_datetime:
                raise interface.Invalid(
                    _(u"Expiration datetime cannot be before publication."))


class VersionedObjectPublicationInformation(grok.Adapter):
    grok.context(IVersionedObject)
    grok.implements(IPublicationInformation)

    @apply
    def expiration_datetime():

        def getter(self):
            return self.context.get_unapproved_version_expiration_datetime()

        def setter(self, datetime):
            if datetime is None:
                return
            self.context.set_unapproved_version_expiration_datetime(
                DateTime(datetime))

        return property(getter, setter)

    @apply
    def publication_datetime():

        def getter(self):
            value = self.context.get_unapproved_version_publication_datetime()
            if value is None:
                # This will make sure we have the default value
                raise KeyError('publication_datetime')
            return value

        def setter(self, datetime):
            self.context.set_unapproved_version_publication_datetime(
                DateTime(datetime))

        return property(getter, setter)


class ContainerPublicationInformation(grok.Adapter):
    grok.context(IContainer)
    grok.implements(IPublicationInformation)

    @property
    def expiration_datetime(self):
        raise KeyError('expiration_datetime')

    @property
    def publication_datetime(self):
        raise KeyError('publication_datetime')


class ApprovalForms(silvaforms.SMISubFormGroup):
    grok.context(IVersionedObject)
    grok.view(Publish)
    grok.order(10)

    def available(self):
        if not self.context.get_unapproved_version():
            return False
        return super(ApprovalForms, self).available()


class PublicationAction(EditAction):
    """An author request approval for a content.
    """

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            return silvaforms.FAILURE

        content_data = form.getContentData()
        content = content_data.getContent()

        self.applyData(form, content_data, data)
        return self.execute(form, content, data)


class RequestApprovalAction(PublicationAction):
    title = _('Request approval')

    def execute(self, form, content, data):
        message = data.getWithDefault('message')
        try:
            IPublicationWorkflow(content).request_approval(message)
        except VersioningError as e:
            form.send_message(e.reason, type='error')
            return silvaforms.FAILURE
        form.send_message(_(u"Approval requested."), type='feedback')
        return silvaforms.SUCCESS


class RequestApprovalForm(silvaforms.SMISubForm):
    grok.context(IVersionedObject)
    grok.view(ApprovalForms)
    grok.order(20)

    label = _('Request approval for new version')
    fields = autofields.FieldsCollector(IPublicationFields)
    actions = silvaforms.Actions(RequestApprovalAction())
    ignoreContent = False
    ignoreRequest = False

    def available(self):
        """ This form do not show if there is no unapproved version or if
        the current user can directly approve the content
        """
        if checkPermission('silva.ApproveSilvaContent', self.context):
            return False
        return not self.context.is_approval_requested()


class PublishAction(PublicationAction):
    title = _('Publish now')

    def available(self, form):
        return bool(checkPermission('silva.ApproveSilvaContent', form.context))

    def execute(self, form, content, data):
        # Date from the form should already be set (see PublicationAction)
        try:
            IPublicationWorkflow(form.context).publish()
        except VersioningError as e:
            form.send_message(e.reason, type='error')
            return silvaforms.FAILURE
        form.send_message(_(u"Version published."), type='feedback')
        return silvaforms.SUCCESS


class ApproveAction(PublicationAction):
    title = _('Approve for future')

    def available(self, form):
        return bool(checkPermission('silva.ApproveSilvaContent', form.context))

    def execute(self, form, content, data):
        # Date from the form should already be set (see PublicationAction)
        try:
            IPublicationWorkflow(form.context).approve()
        except VersioningError as e:
            form.send_message(e.reason, type='error')
            return silvaforms.FAILURE
        form.send_message(_(u"Version approved."), type='feedback')
        return silvaforms.SUCCESS


class PublicationForm(silvaforms.SMISubForm):
    grok.context(IVersionedObject)
    grok.view(ApprovalForms)
    grok.order(20)

    fields = autofields.FieldsCollector(IPublicationFields)
    dataManager = autofields.FieldsDataManager()
    actions = silvaforms.Actions(
        ApproveAction(identifier='approve'),
        PublishAction(identifier='publish-now'))
    ignoreContent = False
    ignoreRequest = False

    label = _(u'Publish new version')

    def available(self):
        return bool(checkPermission('silva.ApproveSilvaContent', self.context))


approval_status_vocabulary = SimpleVocabulary([
        SimpleTerm(value='request', title=_(u'Request publication')),
        SimpleTerm(value='reject', title=_(u'Reject publication')),
        SimpleTerm(value='withdraw', title=_(u'Withdraw publication')),
        SimpleTerm(value='approve', title=_(u'Approve publication'))])


class IRequestForApprovalStatusMessage(interface.Interface):
    user_id = schema.TextLine(title=u"User")
    date = schema.Datetime(title=u"Date")
    message = schema.Text(title=u"Message")
    status = schema.Choice(title=u"Action", source=approval_status_vocabulary)


class RequestForApprovalStatusDisplayForm(silvaforms.SMISubTableForm):
    grok.context(IVersionedObject)
    grok.view(ApprovalForms)
    grok.order(50)

    label = _(u"Request approval history")
    tableFields = silvaforms.Fields(IRequestForApprovalStatusMessage)
    mode = silvaforms.DISPLAY
    ignoreContent = False
    ignoreRequest = True
    emptyDescription = _(u"There is no history.")
    batchSize = 5

    def widgetsByKey(self, line):
        result = {}
        for key in line.keys():
            result[key.split('.')[-1]] = line[key]
        return result

    def getItems(self):
        version_id = self.context.get_unapproved_version()
        version = self.context._getOb(version_id)
        messages = list(IRequestForApprovalStatus(version).messages)
        messages.reverse()
        return messages


class DefaultRequestApprovalFields(autofields.AutoFields):
    autofields.group(IPublicationFields)
    autofields.order(10)
    fields = silvaforms.Fields(IPublicationInformation)
    fields['publication_datetime'].defaultValue = lambda f: datetime.now()
    fields['publication_datetime'].required = lambda f: checkPermission(
        'silva.ApproveSilvaContent', f.context)


class IRequestApprovalMessage(interface.Interface):
    message = schema.Text(
        title=_(u'Message'),
        description=_(u'This message will remain visible in the status'
                      u' screen for other authors and editors.'),
        default=u'',
        required=False)


class RequestApprovalMessagePublication(object):
    grok.context(IVersioning)
    grok.implements(IRequestApprovalMessage)

    def get_message(self):
        return None

    def set_message(self):
        pass

    message = property(get_message, set_message)


class RequestApprovalMessageFields(autofields.AutoFields):
    autofields.group(IPublicationFields)
    autofields.order(20)
    grok.view(RequestApprovalForm)
    fields = silvaforms.Fields(IRequestApprovalMessage)


class PendingApprovalRequestForm(silvaforms.SMISubForm):
    grok.baseclass()
    grok.context(IVersionedObject)
    grok.view(ApprovalForms)
    grok.order(30)

    def available(self):
        """ This form do not show if there is no pending approval version or if
        the current user can directly approve the content
        """
        return self.context.is_approval_requested()


class IWithdrawalMessage(interface.Interface):
    message = schema.Text(title=_('Add message on withdrawal'),
            description=_('The message may be viewed by local users.'),
            default=u'',
            required=False)


class WithdrawApprovalRequestForm(PendingApprovalRequestForm):
    fields = silvaforms.Fields(IWithdrawalMessage)
    label = _(u'Withdraw approval request')

    def available(self):
        if checkPermission('silva.ApproveSilvaContent', self.context):
            return False
        return super(WithdrawApprovalRequestForm, self).available()

    @silvaforms.action(
        _(u'Withdraw approval request'),
        identifier='withdraw')
    def withdraw(self):
        data, errors = self.extractData()
        if errors:
            return silvaforms.FAILURE
        message = data.getWithDefault('message')
        try:
            IPublicationWorkflow(self.context).withdraw_request(message)
        except VersioningError as e:
            self.send_message(e.reason, type='error')
            return silvaforms.FAILURE
        self.send_message('Withdrew request for approval.', type='feedback')
        return silvaforms.SUCCESS


class IRejectionMessage(interface.Interface):
    message = schema.Text(title=_(u'Add message on rejection'),
            description=_(u'The message may be viewed by local users.'),
            default=u'',
            required=False)


class RejectApprovalRequestForm(PendingApprovalRequestForm):
    """ Reject approval from an author by and editor.
    """
    fields = silvaforms.Fields(IRejectionMessage)
    label = _(u'Reject approval request')

    def available(self):
        return checkPermission('silva.ApproveSilvaContent', self.context) \
            and super(RejectApprovalRequestForm, self).available()

    @silvaforms.action(
        _('Reject approval request'),
        identifier='reject')
    def reject(self):
        data, errors = self.extractData()
        if errors:
            return silvaforms.FAILURE
        message = data.getWithDefault('message')
        try:
            IPublicationWorkflow(self.context).reject_request(message)
        except VersioningError as e:
            self.send_message(e.reason, type='error')
            return silvaforms.FAILURE
        self.send_message('Rejected request for approval.', type='feedback')
        return silvaforms.SUCCESS


class ManualCloseForm(silvaforms.SMISubForm):
    """ Close the public version.
    """
    grok.context(IVersionedObject)
    grok.view(Publish)
    grok.order(40)

    label = _(u'Manual close')
    description = _(u'If necessary, the published version of this '
        u'content can be manually closed (taken offline). '
        u'It will also be removed from the public index.')

    def available(self):
        return checkPermission('silva.ApproveSilvaContent', self.context) and \
            self.context.get_public_version() is not None

    @silvaforms.action(
        _(u'Close published version'),
        identifier='close',
        implements=IRemoverAction)
    def close(self):
        try:
            IPublicationWorkflow(self.context).close()
        except VersioningError as e:
            self.send_message(e.reason, type='error')
            return silvaforms.FAILURE
        self.send_message('Version closed.', type='feedback')
        return silvaforms.SUCCESS


class PublishLastClosedForm(silvaforms.SMISubForm):
    """ Publish the last closed version.
    """
    grok.context(IVersionedObject)
    grok.view(Publish)
    grok.order(41)

    actions = silvaforms.Actions(PublishAction(identifier="publish-now"))

    label = _(u"Publish closed version")
    description = _(u"Make the last closed version public.")

    def available(self):
        return checkPermission('silva.ApproveSilvaContent', self.context) and \
            self.context.get_last_closed_version() is not None and \
            self.context.get_public_version() is None and \
            self.context.get_unapproved_version() is None



# -*- coding: utf-8 -*-
# Copyright (c) 2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings


class AuthorFileTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            with self.layer.open_fixture('silva.png') as stream:
                factory.manage_addFile('logo', 'Silva Logo', stream)

    def test_file_listing(self):
        """You can add a file with Selenium (since the upload button
        doesn't work), however we can check the listing and edit view.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn('Silva File', browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva File'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva File"])
        self.assertEqual(browser.inspect.toolbar, [])

        form = browser.inspect.form['Add a Silva File']
        self.assertIn('Id', form.fields)
        self.assertIn('Title', form.fields)
        self.assertEqual(form.actions, ['Cancel', 'Save'])
        self.assertEqual(form.actions['Cancel'].click(), 200)

        self.assertEqual(
            browser.inspect.listing_groups,
            [u'Structural items', u'Assets'])
        self.assertEqual(browser.inspect.listing_groups['Assets'].click(), 200)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Silva Logo',
              'identifier': 'logo',
              'author': u'editor'}])
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(browser.inspect.listing[0].goto_dropdown.click(), 200)
        self.assertEqual(
            browser.inspect.listing[0].goto_actions,
            ['Preview', 'Properties'])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Inspect newly created content
        self.assertEqual(browser.inspect.title, u'Silva Logo')
        self.assertEqual(
            browser.inspect.tabs,
            ['Edit', 'Preview', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.views, ['View...'])
        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        self.assertEqual(browser.inspect.form, ['Edit file content'])
        form = browser.inspect.form['Edit file content']
        self.assertIn('Title', form.fields)
        self.assertIn('File', form.fields)
        self.assertEqual(form.actions, ['Back', 'Save changes'])

        # An action let you directly request approval
        self.assertEqual(browser.inspect.toolbar, [])

        # We go through the tabs.
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Edit'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our ghost and the original document.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Silva Logo',
              'identifier': 'logo',
              'author': u'editor'}])

        # Select it
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename'])

        self.assertEqual(browser.inspect.toolbar['Delete'].click(), 200)
        # Content is not deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Silva Logo',
              'identifier': 'logo',
              'author': u'editor'}])
        self.assertEqual(browser.inspect.dialog, ['Confirm deletion'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(browser.inspect.listing, [])
        browser.macros.assertFeedback(u'Deleted "Silva Logo".')


class EditorFileTestCase(AuthorFileTestCase):
    user = 'editor'


class ChiefEditorFileTestCase(EditorFileTestCase):
    user = 'chiefeditor'


class ManagerFileTestCase(ChiefEditorFileTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AuthorFileTestCase))
    suite.addTest(unittest.makeSuite(EditorFileTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorFileTestCase))
    suite.addTest(unittest.makeSuite(ManagerFileTestCase))
    return suite




# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from datetime import datetime
import mimetypes

from five import grok
from zope import schema
from zope.component import getAdapter, getAdapters
from zope.interface import Interface
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from silva.core.interfaces import IContainer, IGhostAware
from silva.core.interfaces import IContentExporter, IDefaultContentExporter
from silva.core.smi.content.container import ContainerMenu, Container
from silva.translations import translate as _
from silva.ui.menu import MenuItem
from silva.ui.rest import RESTResult
from zeam.form import silva as silvaforms
from zeam.form.ztk.interfaces import IFormSourceBinder

from zExceptions import BadRequest


@grok.provider(IFormSourceBinder)
def exporters_vocabulary(form):
    """List available exporter (this works only with Zeam form).
    """
    terms = []
    for token, adapter in form.exporters:
        terms.append(SimpleTerm(value=token, title=adapter.name))
    return SimpleVocabulary(terms)

def default_format(form):
    if len(form.exporters):
        return form.exporters[0][0]
    return None


class IExportFields(Interface):
    export_format = schema.Choice(
        source=exporters_vocabulary,
        title=_(u"Select an export format"),
        description=_(u"Select the format which will be used for the export."))


class ExportAction(silvaforms.LinkAction):
    grok.implements(silvaforms.IDefaultAction)

    title = _(u"Export")
    description = _(u"export content")
    accesskey = 'ctrl+e'

    def __call__(self, form):
        data, errors = form.extractData()
        if errors:
            raise BadRequest('invalid export parameters')

        options = {'request': form.request}
        for key in data:
            options[key] = data.getWithDefault(key)

        exporter = getAdapter(
            form.context, IContentExporter, name=options['export_format'])
        filename = '%s_export_%s.%s' % (
            form.context.id,
            datetime.now().strftime("%Y-%m-%d"),
            exporter.extension)
        output = exporter.export(**options)

        def payload(rest):
            content_type, content_encoding = mimetypes.guess_type(filename)
            if not content_type:
                content_type = 'application/octet-stream'
            response = rest.response
            response.setHeader('Content-Type', content_type)
            if content_encoding:
                response.setHeader('Content-Encoding', content_encoding)
            response.setHeader(
                'Content-Disposition', 'attachment;filename=%s' % filename)
            return output

        raise RESTResult(payload)


class ExportForm(silvaforms.SMIForm):
    """Export form for containers.
    """
    grok.adapts(Container, IContainer)
    grok.require('silva.ManageSilvaContentSettings')
    grok.name('export')

    label = _(u"Export")

    fields = silvaforms.Fields(IExportFields)
    fields['export_format'].mode = 'radio'
    fields['export_format'].defaultValue = default_format
    actions = silvaforms.Actions(silvaforms.CancelAction(), ExportAction())

    postOnly = False
    ignoreContent=True
    ignoreRequest=False

    def update(self):
        exporters = []
        default = None
        options = silvaforms.Fields()
        for adapter in getAdapters((self.context,), IContentExporter):
            if IDefaultContentExporter.providedBy(adapter[1]):
                assert default is None, \
                    "There are two defaults content exporter"
                default = adapter
            else:
                exporters.append(adapter)
            options.extend(adapter[1].options)
        self.exporters = [default,] + exporters
        self.fields += options


class ExportMenu(MenuItem):
    grok.adapts(ContainerMenu, IContainer)
    grok.order(81)
    grok.require('silva.ManageSilvaContentSettings')

    name = _(u'Export')
    screen = ExportForm

    def available(self):
        return not IGhostAware.providedBy(self.content)




# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from AccessControl import getSecurityManager

from five import grok
from z3c.schema.email import RFC822MailAddress
from zeam.form import silva as silvaforms
from zope import schema
from zope.interface import Interface
from zope.component import getUtility, queryUtility
from zope.component.hooks import getSite
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.traversing.browser import absoluteURL

from silva.core.interfaces import ISilvaObject, IEditableMember
from silva.core.interfaces.adapters import ILanguageProvider
from silva.core.interfaces.auth import IAuthorizationManager
from silva.core.services.interfaces import IMemberService
from silva.translations import translate as _
from silva.ui.interfaces import IUIService 
from silva.ui.menu import UserMenu, LinkMenuItem, ExpendableMenuItem


@grok.provider(IContextSourceBinder)
def language_source(context):
    langs = [
        SimpleTerm(
            value=None,
            token='none',
            title=_(u'Use browser language setting'))]
    languages = ILanguageProvider(context.REQUEST)
    codes = languages.getAvailableLanguages()
    # If the current language is not in the available list, we
    # still add it in order to prevent vocabularies to fail
    preferred = languages.getPreferredLanguage()
    if preferred not in codes:
        codes.append(preferred)
    codes.sort()
    for lang in codes:
        langs.append(SimpleTerm(
            value=lang, token=lang, title=languages.getLanguageName(lang)))
    return SimpleVocabulary(langs)


class IUserInfo(Interface):
    userid = schema.TextLine(
        title=_(u"User identifier"),
        description=_(
            u"User identifier used to authenticate."),
        required=False)
    fullname = schema.TextLine(
        title=_(u"Fullname"),
        description=_(u"User full name."),
        required=True)
    email = RFC822MailAddress(
        title=_(u"Email address"),
        description=_(u"Contact email address for the user."),
        required=True)
    language = schema.Choice(
        title=_(u"Preferred language"),
        description=_(u"Language to use in the Silva interface."),
        source=language_source,
        required=False)


class UserDataManager(silvaforms.ObjectDataManager):

    def __init__(self, content, request):
        super(UserDataManager, self).__init__(content)
        self.language = ILanguageProvider(request)

    def get(self, key):
        if key in ('userid', 'email', 'fullname'):
            return getattr(self.content, key)()
        if key == 'language':
            return self.language.getPreferredLanguage()
        raise KeyError(key)

    def set(self, key, value):
        if key in ('email', 'fullname'):
            if IEditableMember.providedBy(self.content):
                return getattr(self.content, 'set_' + key)(value)
        if key == 'language':
            return self.language.setPreferredLanguage(value)


class UserPreferences(silvaforms.RESTPopupForm):
    grok.context(ISilvaObject)
    grok.name('silva.core.smi.userpreferences')

    label = _(u"User preferences")
    fields = silvaforms.Fields(IUserInfo)
    fields['userid'].mode = silvaforms.DISPLAY
    ignoreContent = False
    actions = silvaforms.Actions(
        silvaforms.CancelAction(),
        silvaforms.EditAction())

    def update(self):
        service = getUtility(IMemberService)
        member = service.get_member(getSecurityManager().getUser().getId())
        if not IEditableMember.providedBy(member):
            self.fields['fullname'].mode = silvaforms.DISPLAY
            self.fields['email'].mode = silvaforms.DISPLAY
        self.user = UserDataManager(member, self.request)

    def getContentData(self):
        return self.user


class UserSettingsMenu(ExpendableMenuItem):
    grok.adapts(UserMenu, Interface)
    logo = 'tab-user'

    @property
    def description(self):
        content = self.content.get_silva_object()
        authorization = IAuthorizationManager(content, None)
        if authorization is None:
            return None
        information = authorization.get_authorization()
        return '{0} ({1})'.format(information.name, information.acquired_role)


class PreferencesMenu(LinkMenuItem):
    grok.adapts(UserSettingsMenu, Interface)
    grok.order(5)
    name = _('User Preferences')
    icon = 'preferences'
    trigger = 'form-popup'

    def get_url(self, context, request):
        content = context.get_silva_object()
        return '{0}/++rest++silva.core.smi.userpreferences'.format(
            absoluteURL(content, request))


class ManageMenu(LinkMenuItem):
    grok.adapts(UserSettingsMenu, Interface)
    grok.order(20)
    grok.require('zope2.ViewManagementScreens')
    name = _('Manage...')
    icon = 'manage'

    def available(self):
        service = queryUtility(IUIService)
        return service is not None and service.smi_link_zmi

    def get_url(self, context, request):
        return '{0}/manage_workspace'.format(absoluteURL(context, request))


class ServicesMenu(LinkMenuItem):
    grok.adapts(UserSettingsMenu, Interface)
    grok.order(25)
    grok.require('zope2.ViewManagementScreens')
    name = _('Services...')
    icon = 'services'

    def available(self):
        service = queryUtility(IUIService)
        return service is not None and service.smi_link_zmi

    def get_url(self, context, request):
        return '{0}/manage_services'.format(absoluteURL(getSite(), request))


class LogoutMenu(LinkMenuItem):
    grok.adapts(UserSettingsMenu, Interface)
    grok.order(50)
    name = _('Logout')
    icon = 'logout'
    target = '_self'

    def get_url(self, context, request):
        return '{0}/service_members/logout'.format(
            absoluteURL(context.get_root(), request))



# -*- coding: utf-8 -*-
# Copyright (c) 2011-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from silva.core.interfaces import ISiteManager
from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings


class ReaderPublicationTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addPublication('site', 'Site')

    def test_publication_access(self):
        """A reader cannot create a folder, however he can look into it.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('reader')
        self.assertEqual(browser.inspect.title, u'root')
        self.assertEqual(browser.inspect.tabs, ['Content', 'Preview', ])
        self.assertEqual(browser.inspect.activetabs, ['Content'])
        self.assertEqual(browser.inspect.views, ['View...'])
        self.assertEqual(browser.inspect.toolbar, [])

        # We should see our folder.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Site', 'identifier': 'site', 'author': 'editor'}])
        # Select the folder.
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(browser.inspect.toolbar, ['Copy'])
        self.assertEqual(browser.inspect.listing[0].goto_dropdown.click(), 200)
        self.assertEqual(browser.inspect.listing[0].goto_actions, ['Preview'])

        # Go inside the publication.
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.title, u'Site')
        self.assertEqual(browser.inspect.listing, [])


class AuthorPublicationTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'
    publish = False
    access = False

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')

    def test_publication_add_and_listing(self):
        """Create a Publication check its tabs and actions and delete it.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addPublication('site', 'Site')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertNotIn(
            'Silva Publication',
            browser.inspect.tabs['Add'].entries)

        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Site', 'identifier': 'site', 'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.title, u'Site')
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.views, ['View...'])
        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Content'])
        self.assertEqual(browser.inspect.listing, [])

        # We go through the tabs
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Content'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our publication.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Site', 'identifier': 'site', 'author': 'editor'}])
        # Select the folder
        self.assertEqual(
            browser.inspect.listing[0].identifier.click(),
            200)
        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename'])
        self.assertEqual(
            browser.inspect.listing[0].goto_dropdown.click(),
            200)
        self.assertEqual(
            browser.inspect.listing[0].goto_actions,
            ['Preview', 'Properties'])

        # Delete the publication
        self.assertEqual(
            browser.inspect.toolbar['Delete'].click(),
            200)
        # Publication is not deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Site', 'identifier': 'site', 'author': 'editor'}])
        self.assertEqual(
            browser.inspect.dialog,
            [{'title': 'Confirm deletion'}])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(
            browser.inspect.listing,
            [])
        browser.macros.assertFeedback(u'Deleted "Site".')

    def test_publication_convert(self):
        """Test folder to publication conversion.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addPublication('publication', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'publication'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.title, u'Data')
        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertNotIn({'title': u'Container type'}, browser.inspect.form)


class EditorPublicationTestCase(AuthorPublicationTestCase):
    user = 'editor'

    def test_publication_add_and_listing(self):
        """Create a folder check its tabs and actions and delete it.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn(
            'Silva Publication',
            browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva Publication'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva Publication"])

        add_form = browser.inspect.form['Add a Silva Publication']
        add_form.fields['id'].value = 'site'
        add_form.fields['title'].value = 'Site'
        self.assertEqual(add_form.actions, ['Cancel', 'Save'])
        self.assertEqual(add_form.actions['Save'].click(), 200)
        browser.macros.assertFeedback(u"Added Silva Publication.")

        self.assertEqual(browser.inspect.title, u"Site")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.views, ['View...'])
        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Content'])
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Site', 'identifier': 'index', 'author': self.user}])

        # We go through the tabs
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Content'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our folder.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Site', 'identifier': 'site', 'author': self.user}])
        # Select the folder
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename', 'Publish'])
        self.assertEqual(
            browser.inspect.listing[0].goto_dropdown.click(),
            200)
        self.assertEqual(
            browser.inspect.listing[0].goto_actions,
            ['Preview', 'Properties'] + (['Access'] if self.access else []))

        # Delete the folder
        self.assertEqual(
            browser.inspect.toolbar['Delete'].click(),
            200)
        # Folder is yet deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Site', 'identifier': 'site', 'author': self.user}])
        self.assertEqual(
            browser.inspect.dialog,
            [{'title': 'Confirm deletion'}])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(
            browser.inspect.listing,
            [])
        browser.macros.assertFeedback(u'Deleted "Site".')

    def test_publication_convert(self):
        """Test folder to publication conversion.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addPublication('publication', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'publication'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.title, u'Data')
        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertIn({'title': u'Container type'}, browser.inspect.form)

        convert = browser.inspect.form['Container type']
        self.assertEqual(convert.title, 'Container type')
        self.assertEqual(convert.form.inspect.fields, [])
        self.assertEqual(convert.actions, ['Convert to folder', 'Make local site'])
        self.assertEqual(convert.actions['Convert to folder'].click(), 200)
        browser.macros.assertFeedback("Changed into folder.")

        self.assertEqual(browser.inspect.title, u'Data')
        convert = browser.inspect.form['Container type']
        self.assertEqual(convert.title, 'Container type')
        self.assertEqual(convert.form.inspect.fields, [])
        self.assertNotIn('Convert to folder', convert.actions)

    def test_publication_localsite(self):
        """Test activating a local site on a publication.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addPublication('publication', 'Data')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'publication'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)
        self.assertEqual(browser.inspect.title, u'Data')
        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertIn({'title': u'Container type'}, browser.inspect.form)
        # The publication is not a local site by default.
        self.assertFalse(ISiteManager(self.root.publication).is_site())

        # You can activate the local site.
        convert = browser.inspect.form['Container type']
        self.assertEqual(convert.title, 'Container type')
        self.assertEqual(convert.form.inspect.fields, [])
        self.assertEqual(convert.actions, ['Convert to folder', 'Make local site'])
        self.assertEqual(convert.actions['Make local site'].click(), 200)
        browser.macros.assertFeedback("Local site activated.")
        # And the publication is now a local site.
        self.assertTrue(ISiteManager(self.root.publication).is_site())

        # After you activated the site, you can only deactivate it.
        self.assertIn({'title': u'Container type'}, browser.inspect.form)
        convert = browser.inspect.form['Container type']
        self.assertEqual(convert.title, 'Container type')
        self.assertEqual(convert.form.inspect.fields, [])
        self.assertEqual(convert.actions, ['Remove local site'])
        self.assertEqual(convert.actions['Remove local site'].click(), 200)
        browser.macros.assertFeedback("Local site deactivated.")
        # And the publication is no longer a local site.
        self.assertFalse(ISiteManager(self.root.publication).is_site())

        # Once deactivated, you can reactivate it.
        self.assertIn({'title': u'Container type'}, browser.inspect.form)
        convert = browser.inspect.form['Container type']
        self.assertEqual(convert.title, 'Container type')
        self.assertEqual(convert.form.inspect.fields, [])
        self.assertEqual(convert.actions, ['Convert to folder', 'Make local site'])

    def test_publication_localsite_active(self):
        """You cannot unactivate a local site if there are services
        registered in it.
        """
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addPublication('publication', 'Data')
            ISiteManager(self.root.publication).make_site()
            factory = self.root.publication.manage_addProduct['silva.core.layout']
            factory.manage_addCustomizationService('service_customization')

        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)
        self.assertEqual(
            browser.inspect.listing,
            [{'title': 'Data', 'identifier': 'publication'}])
        self.assertEqual(
            browser.inspect.listing[0].goto.click(),
            200)
        self.assertEqual(browser.inspect.title, u'Data')
        self.assertIn('Settings', browser.inspect.tabs)
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertIn({'title': u'Container type'}, browser.inspect.form)

        # You should have the local site form, and the site should be active.
        convert = browser.inspect.form['Container type']
        self.assertEqual(convert.title, 'Container type')
        self.assertEqual(convert.form.inspect.fields, [])
        self.assertEqual(convert.actions, ['Remove local site'])
        self.assertEqual(convert.actions['Remove local site'].click(), 200)
        browser.macros.assertError("Still have registered services.")

        # The site was not removed.
        self.assertTrue(ISiteManager(self.root.publication).is_site())


class ChiefEditorPublicationTestCase(EditorPublicationTestCase):
    user = 'chiefeditor'
    access = True


class ManagerPublicationTestCase(ChiefEditorPublicationTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReaderPublicationTestCase))
    suite.addTest(unittest.makeSuite(AuthorPublicationTestCase))
    suite.addTest(unittest.makeSuite(EditorPublicationTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorPublicationTestCase))
    suite.addTest(unittest.makeSuite(ManagerPublicationTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from silva.core.interfaces import IIndexer
from zope.interface.verify import verifyObject

from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings


class ReaderIndexerTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addIndexer('indexer', 'Indexer')

    def test_indexer_access(self):
        """A reader cannot create an indexer, however he look at it.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('reader')
        self.assertEqual(browser.inspect.title, u'root')

        # We should see our indexer
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Indexer',
              'identifier': 'indexer',
              'author': 'editor'}])
        # Select it.
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(browser.inspect.toolbar, ['Copy'])
        self.assertEqual(browser.inspect.listing[0].goto_dropdown.click(), 200)
        self.assertEqual(browser.inspect.listing[0].goto_actions, ['Preview'])

        # Go on it. We arrive on the edit tab, where we cannot do a thing.
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)
        self.assertEqual(browser.inspect.title, u'Indexer')
        self.assertEqual(browser.inspect.toolbar, [])
        self.assertEqual(browser.inspect.tabs, ['Edit', 'Preview'])
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        self.assertEqual(browser.inspect.views, ['View...'])
        self.assertEqual(browser.inspect.form, ['Update Silva Indexer'])
        edit_form = browser.inspect.form['Update Silva Indexer']
        self.assertEqual(edit_form.fields, [])
        self.assertEqual(edit_form.actions, ['Back'])

        # We go through the tabs.
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Edit'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])


class AuthorIndexerTestCase(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addIndexer('indexer', 'Indexer')

    def test_indexer_update(self):
        """An author cannot an an indexer, but can update it.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login('author')

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].name.click(), 200)
        self.assertNotIn('Silva Indexer', browser.inspect.tabs['Add'].entries)

        # We should see our indexer
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Indexer',
              'identifier': 'indexer',
              'author': 'editor'}])
        # Visit it.
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        self.assertEqual(browser.inspect.title, u'Indexer')
        self.assertEqual(
            browser.inspect.tabs,
            ['Edit', 'Preview', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.views, ['View...'])
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        self.assertEqual(browser.inspect.toolbar, [])
        # We are on contents
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])


class EditorIndexerTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'editor'

    def setUp(self):
        self.root = self.layer.get_application()

    def test_indexer_roundtrip(self):
        """An editor can an an indexer, and update it.
        """
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(
            browser.inspect.tabs,
            ['Content', 'Preview', 'Add', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.tabs['Add'].open.click(), 200)
        self.assertIn('Silva Indexer', browser.inspect.tabs['Add'].entries)
        self.assertEqual(
            browser.inspect.tabs['Add'].entries['Silva Indexer'].click(),
            200)
        self.assertEqual(browser.inspect.form, ["Add a Silva Indexer"])
        self.assertEqual(browser.inspect.toolbar, [])

        # Add an indexer
        add_form = browser.inspect.form['Add a Silva Indexer']
        add_form.fields['id'].value = 'indexer'
        add_form.fields['title'].value = u'Indexer'
        self.assertEqual(add_form.actions, ['Cancel', 'Save'])
        self.assertEqual(add_form.actions['Save'].click(), 200)
        browser.macros.assertFeedback(u"Added Silva Indexer.")
        # We should now have an indexer with the id 'indexer'
        self.assertTrue(verifyObject(IIndexer, self.root._getOb('indexer', None)))

        # Inspect new created content
        self.assertEqual(browser.inspect.title, u'Indexer')
        self.assertEqual(
            browser.inspect.tabs,
            ['Edit', 'Preview', 'Properties', 'Settings'])
        self.assertEqual(browser.inspect.views, ['View...'])
        # We are on contents
        self.assertEqual(browser.inspect.activetabs, ['Edit'])
        self.assertEqual(browser.inspect.toolbar, [])

        # Check the edit form. There is only an update button.
        self.assertEqual(browser.inspect.form, ['Update Silva Indexer'])
        edit_form = browser.inspect.form['Update Silva Indexer']
        self.assertEqual(edit_form.fields, [])
        self.assertEqual(edit_form.actions, ['Back', 'Update index'])
        self.assertEqual(edit_form.actions['Update index'].click(), 200)
        browser.macros.assertFeedback(
            u'Index content have been successfully updated.')

        # We go through the tabs.
        self.assertEqual(browser.inspect.tabs['Preview'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Preview'])
        self.assertEqual(browser.inspect.tabs['Settings'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Settings'])
        self.assertEqual(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Properties'])
        self.assertEqual(browser.inspect.tabs['Edit'].name.click(), 200)
        self.assertEqual(browser.inspect.activetabs, ['Edit'])

        # Go up a level.
        self.assertEqual(browser.inspect.parent.click(), 200)
        self.assertEqual(browser.inspect.title, u"root")
        self.assertEqual(browser.inspect.activetabs, ['Content'])

        # We should see our auto toc
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Indexer',
              'identifier': 'indexer',
              'author': self.user}])
        # Select it
        self.assertEqual(browser.inspect.listing[0].identifier.click(), 200)
        self.assertEqual(
            browser.inspect.toolbar,
            ['Cut', 'Copy', 'Delete', 'Rename'])
        self.assertEqual(browser.inspect.listing[0].goto_dropdown.click(), 200)
        self.assertEqual(
            browser.inspect.listing[0].goto_actions,
            ['Preview', 'Properties'])

        # Delete the indexer
        self.assertEqual(browser.inspect.toolbar['Delete'].click(), 200)
        # Content is not deleted, you have to confirm the deletion first.
        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Indexer', 'identifier': 'indexer'}])
        self.assertEqual(
            browser.inspect.dialog,
            [{'title': 'Confirm deletion'}])
        self.assertEqual(
            browser.inspect.dialog[0].buttons,
            ['Cancel', 'Continue'])
        self.assertEqual(
            browser.inspect.dialog[0].buttons['Continue'].click(),
            200)
        self.assertEqual(browser.inspect.listing, [])
        browser.macros.assertFeedback(u'Deleted "Indexer".')
        self.assertIs(self.root._getOb('indexer', None), None)


class ChiefEditorIndexerTestCase(EditorIndexerTestCase):
    user = 'chiefeditor'


class ManagerIndexerTestCase(ChiefEditorIndexerTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReaderIndexerTestCase))
    suite.addTest(unittest.makeSuite(AuthorIndexerTestCase))
    suite.addTest(unittest.makeSuite(EditorIndexerTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorIndexerTestCase))
    suite.addTest(unittest.makeSuite(ManagerIndexerTestCase))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2002-2013 Infrae. All rights reserved.
# See also LICENSE.txt

import unittest

from zope.component import getUtility
from silva.core.interfaces import IPublicationWorkflow
from silva.core.services.interfaces import IMetadataService

from Products.Silva.testing import FunctionalLayer, Transaction
from Products.Silva.ftesting import smi_settings


class AuthorMetadataTestCase(unittest.TestCase):
    layer = FunctionalLayer
    user = 'author'

    def setUp(self):
        self.root = self.layer.get_application()
        self.layer.login('editor')
        with Transaction():
            factory = self.root.manage_addProduct['Silva']
            factory.manage_addMockupVersionedContent('document', 'Document')
            IPublicationWorkflow(self.root.document).publish()
            IPublicationWorkflow(self.root.document).new_version()

    def test_content_draft_and_published_metadata(self):
        browser = self.layer.get_web_browser(smi_settings)
        browser.login(self.user)

        self.assertEqual(
            browser.inspect.listing,
            [{'title': u'Document',
              'identifier': 'document',
              'author': 'editor'}])
        self.assertEqual(browser.inspect.listing[0].goto.click(), 200)

        # Access publish tab
        self.assertTrue('Properties' in browser.inspect.tabs)
        self.assertTrue(browser.inspect.tabs['Properties'].name.click(), 200)
        self.assertEqual(
            browser.inspect.form,
            ['Editable item properties', 'Public version item properties'])

        # You can edit editable metadata
        # There two times the controls to save the metadata
        form = browser.inspect.form['Editable item properties']
        self.assertEqual(
            form.fields,
            [u'keywords', u'short title', u'contact name', u'title',
             u'contact email', u'subject', u'comment', u'language',
             u'description'])
        self.assertEqual(form.fields['title'].value, 'Document')
        self.assertEqual(form.fields['short title'].value, '')
        self.assertEqual(form.fields['comment'].value, '')
        form.fields['title'].value = 'New document'
        form.fields['short title'].value = 'Document'
        form.fields['comment'].value = u'Ce document a été récement créé.'
        self.assertEqual(
            form.actions,
            ['Cancel', 'Save changes', 'Cancel', 'Save changes'])
        self.assertEqual(
            form.actions.get('Save changes', multiple=True)[0].click(),
            200)
        browser.macros.assertFeedback('Metadata saved.')

        # Metadata changes. So the title
        self.assertEqual(browser.inspect.title, 'New document')
        self.assertEqual(
            browser.inspect.form,
            ['Editable item properties', 'Public version item properties'])

        form = browser.inspect.form['Editable item properties']
        self.assertEqual(
            form.fields,
            [u'keywords', u'short title', u'contact name', u'title',
             u'contact email', u'subject', u'comment', u'language',
             u'description'])
        self.assertEqual(form.fields['title'].value, 'New document')
        self.assertEqual(form.fields['short title'].value, 'Document')
        self.assertEqual(
            form.fields['comment'].value,
            u'Ce document a été récement créé.')

        editable = self.root.document.get_editable()
        metadata = getUtility(IMetadataService).getMetadata(editable)
        self.assertEqual(
            metadata.get('silva-content', 'maintitle'),
            'New document')
        self.assertEqual(
            metadata.get('silva-content', 'shorttitle'),
            'Document')
        self.assertEqual(
            metadata.get('silva-extra', 'comment'),
            u'Ce document a été récement créé.')


class EditorMetadataTestCase(AuthorMetadataTestCase):
    user = 'editor'


class ChiefEditorMetadataTestCase(EditorMetadataTestCase):
    user = 'chiefeditor'


class ManagerMetadataTestCase(ChiefEditorMetadataTestCase):
    user = 'manager'


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AuthorMetadataTestCase))
    suite.addTest(unittest.makeSuite(EditorMetadataTestCase))
    suite.addTest(unittest.makeSuite(ChiefEditorMetadataTestCase))
    suite.addTest(unittest.makeSuite(ManagerMetadataTestCase))
    return suite
